import Renderer from './renderer'
import {
  TextureLoader, Mesh, MeshBasicMaterial, CircleGeometry, 
  Line, BackSide, LineBasicMaterial, Vector3, PointLight, 
  AmbientLight, MeshPhongMaterial, RingGeometry, SphereGeometry, 
  LineLoop, DoubleSide, ShaderMaterial
} from 'three'
import ringVS from './shaders/saturn_ring.vs.glsl'
import ringFS from './shaders/saturn_ring.fs.glsl'

export default class SolarSystem extends Renderer {
  constructor() {
    super()
    this.params = {
      segment: 64,
      universe: {
        radius: 15000,
        texture: './assets/universe.jpg',
      },
      sun: { 
        radius: 50,
        distance: 0,
        selfRotate: 0.001,
        revolution: 0,
        texture: './assets/sun.jpg',
        light: {
          color: 0xffffff,
          intensity: 1,
          ambientColor: 0x090909,
          ambientIntensity: 1,
        }
      },
      planets: {
        mercury: {
          radius: 4,
          distance: 76,
          selfRotate: 0.005,
          revolution: 0.00413,
          texture: './assets/mercury.jpg',
        },
        venus: {
          radius: 8,
          distance: 144,
          selfRotate: 0.01,
          revolution: 0.0016,
          texture: './assets/venus.jpg',
        },
        earth: {
          radius: 9,
          distance: 200,
          selfRotate: 0.01,
          revolution: 0.00098,
          texture: './assets/earth.jpg',
          moon: {
            radius: 0.6,
            distance: 15, // distance to the earth
            selfRotate: 0.005,
            revolution: 0.12, // revolution around the earth
            texture: './assets/moon.jpg',
          },
        },
        mars: {
          radius: 5,
          distance: 304,
          selfRotate: 0.01,
          revolution: 0.0052,
          texture: './assets/mars.jpg',
        },
        jupiter: {
          radius: 51,
          distance: 1040,
          selfRotate: 0.007,
          revolution: 0.0017,
          texture: './assets/jupiter.jpg',
        },
        saturn: {
          radius: 42,
          distance: 1970,
          selfRotate: 0.01,
          revolution: 0.0011,
          texture: './assets/saturn.jpg',
          ring: {
            texture: './assets/saturn_ring.png',
            innerRadius: 45,
            outerRadius: 80,
            xRotate: Math.PI/3,
          }
        },
        uranus: {
          radius: 18.2,
          distance: 3843,
          selfRotate: 0.01,
          revolution: 0.0008,
          texture: './assets/uranus.jpg',
        },
        neptune: {
          radius: 17.6,
          distance: 6011,
          selfRotate: 0.01,
          revolution: 0.0006,
          texture: './assets/neptune.jpg',
        }
      },
      orbit: {
        color: 0xeeeeee,
        transparent: true,
        opacity: 0.2,
      }
    }
    this.sun = null
    this.planets = {
      mercury: {
        planet: null,
        orbit: null,
        revolution: 0,
      },
      venus: {
        planet: null,
        orbit: null,
        revolution: 0,
      },
      earth: {
        planet: null,
        orbit: null,
        revolution: 0,
        moon: {
          planet: null,
          orbit: null,
          revolution: 0,
        }
      },
      mars: {
        planet: null,
        orbit: null,
        revolution: 0,
      },
      jupiter: {
        planet: null,
        orbit: null,
        revolution: 0,
      },
      saturn: {
        planet: null,
        orbit: null,
        revolution: 0,
        ring: null
      },
      uranus: {
        planet: null,
        orbit: null,
        revolution: 0,
      },
      neptune: {
        planet: null,
        orbit: null,
        revolution: 0,
      }
    }
    this.setup()
    this.render()
  }
  setup() {
    // TODO: 1. create a starry sky background (total: 5p)
    //
    // Grading details:
    //   - use the given params `this.params.universe.radius`, and
    //     `this.params.segment` for geometry (1p)
    //   - use the texture from `this.params.universe.texture` as
    //     the background material for the starry sky (1p)
    //   - reduce a possible blur effect occuring for the background
    //     in all directions (1p)
    //   - create the background 3D object (1p)
    //     then add it to the scene graph (1p)















    // TODO: 2. create a sun as the center of the solar system (total: 5p)
    //
    // Grading details:
    //   - use the given params: `this.params.sun.radius`,
    //     and `this.params.segment` for geometry (1p)
    //   - use the texture from `this.params.sun.texture` as
    //     the sun material (1p)
    //   - reduce a possible blur effect occuring at the poles (1p)
    //   - create the 3D sun object and assign to `this.sun` (1p)
    //     then add it to the scene graph (1p)











    // TODO: 3. create a light to simulate the sun light (total: 2p)
    //
    // Grading details:
    //   - create a point light using the given params in `this.params.sun.light` (1p)
    //   - create ambient light using the given params in `this.params.sun.light` (1p)










    // TODO: 4. create the eight planets using provided textures, and
    //          move them with a given distance to the sun (total: 7p)
    //
    // Grading details:
    //   - use the given params in `this.params.planets.<name>.radius`, 
    //     `this.params.segment` for the planet geometry (1p)
    //   - use the texture from `this.params.planets.<name>.texture` 
    //     then create the correct material to show the phong illumination model (1p)
    //   - reduce a possible blur effect occuring at the poles (1p)
    //   - move each planet around the sun with a given distance
    //     (`this.params.planets.<name>.distance`) (1p)
    //   - create each planet and assign them to `this.planets.<name>.planet` (2p)
    //     then add eight of them to the scene graph (1p)




















    // TODO: 5. draw a (circular) orbit for each planet (total: 7p)
    //
    // Grading details:
    //   - use the given params in `this.params.planets.<name>.distance`,
    //     `this.params.segment` for the circle geometry (1p)
    //   - use the given params in `this.params.orbit` then create
    //     the line basic material (1p)
    //   - create each orbit and assign them to `this.planets.<name>.orbit` (2p)
    //     then add eight of them to the scene graph (1p)
    //   - transform each orbit and place it in x-z plane (1p)
    //   - manipulate the vertices to guarantee the orbit is drawn as
    //     a closed circle (1p)




















    // TODO: 6. create the moon and its orbit (total: 6p)
    //
    // Grading details:
    //   - create the moon and assign the 3D moon object to 
    //     `this.planets.earth.moon.planet` using the given params
    //     (`this.params.planets.earth.moon`, `this.params.segment`).
    //     The moon's material should show the phong illumination model (1p)
    //   - set the moon to the correct position with a distance 
    //     (`this.params.planets.earth.moon.distance`) to the earth (1p)
    //   - add the moon to the scene graph (1p)
    //   - create the moon orbit and assign it to `this.planets.earth.moon.orbit`
    //     using the given params `this.params.planets.earth.moon.distance`
    //     and `this.params.orbit` (1p)
    //   - transform the moon's orbit that is located around the earth
    //     in x-z plane, make sure the orbit is drawed in a closed circle (1p)
    //   - add the moon's orbit to the scene graph (1p)


































    // TODO: 7. create the saturn ring and apply corresponing texture
    //          mapping with a customized fragment shader (total: 8p)
    //
    // Grading details:
    //   - use the given params `this.params.planets.saturn.ring` to 
    //     create the saturn ring using ring geometry and a shader material
    //     then assign it to `this.planets.saturn.ring` (1p)
    //   - make sure the saturn ring is rendered on both sides (1p)
    //   - pass the needed uniforms to the customized shaders (1p)
    //   - write a customized vertex shader that passes the position to
    //     fragment shader(src/shaders/saturn_ring.vs.glsl) (1p)
    //   - write a customized fragment shader that performs the UV mapping
    //     (src/shaders/saturn_ring.fs.glsl) (2p)
    //   - transform the saturn ring so that is located around the saturn
    //     and that it is rotating around x axis with the given angle in
    //     `this.params.planets.saturn.ring` and make sure the orbit is
    //     drawn in a closed circle (1p)
    //   - add the saturn ring to the scene graph (1p)






























  }
  update() {
    // TODO: 8. add self rotation to the sun (total: 1p)
    //
    // Grading details:
    //   - use the given params `this.params.sun.selfRotate` and
    //     add self rotation to the sun around the y-axis.
    //     The given parameter is the rotation angle (radians)
    //     of each frame. (1p)







    // TODO: 9. add self rotation to the eight planets (total: 2p)
    //
    // Grading details:
    //   - use the given params `this.params.planets.<name>.selfRotate`
    //     and add self rotation to each planet around y-axis.
    //     The given parameters are the rotation angle (radians)
    //     of each frame. (2p)








    // TODO: 10. add revolution to the eight planets (total: 3p)
    //
    // Grading details:
    //   - use the given params `this.params.planets.<name>.revolution`
    //     and add revolution (2p) to each planet (1p) that rotates 
    //     around the sun. The given parameters are the rotation angle
    //     (radian) of each frame.












    // TODO: 11. add self rotation and revolution to the moon (total: 3p)
    //
    // Grading details:
    //   - use the given params `this.params.planets.earth.moon.selfRotate`
    //     and add self rotation to the moon (1p). The given parameter is
    //     the rotation angle (radians) of each frame.
    //   - use the given params `this.params.planets.earth.moon.revolution`
    //     and add revolution to the moon (1p). The given parameter is
    //     the rotation angle (radians) of each frame.
    //   - move the orbit while the earth is moving by changing
    //     the position of the moon (1p).
















    // TODO: 12. move the saturn ring while saturn is moving (total: 1p)
    //
    // Grading details:
    //   - move the saturn ring while saturn is moving by changing the
    //     position of the saturn ring (1p)








  }
}

new SolarSystem()